Class: Rage::Telemetry::Spans::BroadcastSSEStream
- Inherits:
-
Object
- Object
- Rage::Telemetry::Spans::BroadcastSSEStream
- Defined in:
- lib/rage/telemetry/spans/broadcast_sse_stream.rb
Overview
The sse.stream.broadcast span wraps the process of broadcasting a message to an unbounded SSE stream.
This span is started when Rage::SSE.broadcast is called, and ends when the broadcast operation is complete. See handle for the list of arguments passed to handler methods.
Class Method Summary collapse
Class Method Details
.handle(id:, name:, stream:) ⇒ Object
Note:
Rage automatically detects which parameters your handler method accepts and only passes those parameters. You can omit any of the parameters described here.
16 17 |
# File 'lib/rage/telemetry/spans/broadcast_sse_stream.rb', line 16 def handle(id:, name:, stream:) end |